'use strict';Object.defineProperty(exports, "__esModule", { value: true });exports.createVariants = exports.run = undefined;var _extends = Object.assign || function (target) {for (var i = 1; i < arguments.length; i++) {var source = arguments[i];for (var key in source) {if (Object.prototype.hasOwnProperty.call(source, key)) {target[key] = source[key];}}}return target;};var _chalk = require('chalk');var _chalk2 = _interopRequireDefault(_chalk);
var _createVariants = require('./createVariants');var _createVariants2 = _interopRequireDefault(_createVariants);
var _regeneratorRuntime = require('regenerator-runtime');var _regeneratorRuntime2 = _interopRequireDefault(_regeneratorRuntime);
var _logger = require('./logger');
var _farm = require('./farm');
var _utils = require('./utils');function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}function _asyncToGenerator(fn) {return function () {var gen = fn.apply(this, arguments);return new Promise(function (resolve, reject) {function step(key, arg) {try {var info = gen[key](arg);var value = info.value;} catch (error) {reject(error);return;}if (info.done) {resolve(value);} else {return Promise.resolve(value).then(function (value) {step("next", value);}, function (err) {step("throw", err);});}}return step("next");});};}

/**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Runs the specified webpack configuration in parallel.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {String} configPath The path to the webpack.config.js
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {Object} options
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {Number} [options.maxCallsPerWorker=Infinity] The maximum amount of calls
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   per parallel worker
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {Number} [options.maxConcurrentWorkers=require('os').cpus().length] The
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   maximum number of parallel workers
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {Number} [options.maxConcurrentCallsPerWorker=10] The maximum number of
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   concurrent call per prallel worker
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {Number} [options.maxConcurrentCalls=Infinity] The maximum number of
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   concurrent calls
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {Number} [options.maxRetries=0] The maximum amount of retries
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   on build error
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @param {Function} [callback] A callback to be invoked once the build has
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   been completed
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @return {Promise} A Promise that is resolved once all builds have been
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *   created
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */
const run = exports.run = (() => {var _ref = _asyncToGenerator(function* (configPath) {let paramOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};let callback = arguments[2];
        const options = _extends({
            argv: [],
            colors: _chalk2.default.supportsColor },
        paramOptions);

        const farmOptions = _extends({}, paramOptions);
        const silent = !!options.json;
        let config;

        if (options.colors) {
            process.env.FORCE_COLOR = 1;
        }

        options.argv.unshift(process.execPath, 'parallel-webpack');

        try {
            config = yield (0, _utils.loadConfig)(configPath, options);
        } catch (error) {
            return Promise.reject(error);
        }

        try {
            yield (0, _utils.validateOptions)(farmOptions);
        } catch (error) {
            return Promise.reject(error);
        }

        const workers = (0, _farm.createFarm)(farmOptions);

        const shutdownCallback = function shutdownCallback() {
            if (!silent) {
                (0, _logger.error)(`Forcefully shutting down`);
            }
            workers.end();
            process.exit(1);
        };

        process.on('SIGINT', shutdownCallback);

        const startTime = Date.now();
        const farmPromise = (0, _farm.startFarm)(
        config,
        configPath,
        options,
        workers,
        callback).

        then(function (results) {
            if (!silent) {
                const finishTime = (Date.now() - startTime) / 1000;
                (0, _logger.info)('Finished build after %s seconds', _chalk2.default.blue(finishTime));
            }

            workers.end();
            process.removeListener('SIGINT', shutdownCallback);

            results = results.filter(function (result) {return result;});
            if (results.length) {
                if (callback) {
                    callback(null, results);
                }
                return results;
            }
        }).
        catch(function (err) {
            if (!silent) {
                const finishTime = (Date.now() - startTime) / 1000;
                (0, _logger.error)('Build failed after %s seconds', _chalk2.default.blue(finishTime));
            }

            workers.end();
            process.removeListener('SIGINT', shutdownCallback);

            if (callback) {
                callback(err);
            } else {
                return Promise.reject(err);
            }
        });

        if (!callback) {
            return farmPromise;
        }
    });return function run(_x2) {return _ref.apply(this, arguments);};})();exports.

createVariants = _createVariants2.default;